#!/usr/local/bin/ruby

require "net/http"
require "net/smtp"
require 'open-uri'
require 'rss/1.0'
require 'rss/2.0'
require "kconv"
require "cgi"

$KCODE = "sjis"

class GoogleTranslate

  def initialize line = nil
    @port = 80
    @proxy = nil
    @proxy_port = nil
    @post_site = "translate.google.com"
    @post_uri  = "/translate_t"

    if line
      line = CGI.escape(line)
#      line.gsub!(/([^ \+])(%)/, '\1+\2')
#      line.gsub!(/(%[0-9A-F]{2})([^ \+])/, '\1+\2')
    else
      line = "The quick brown fox jumps over the lazy dog.".tr(' ', '+')
    end
    @post_line = "text=" + line + "&langpair=en%7Cja&hl=ja&ie=UTF-8&oe=SJIS"
  end

  def translate
    res, page = Net::HTTP::new(@post_site, @port, @proxy, @proxy_port).post(@post_uri, @post_line)
    # <textarea name=q ...> "translated text" </textarea> with multipule lines in SJIS
    trans_text = page.sub(/(.*<div id=result_box dir=ltr>)(.+?)(<\/div.*)/m, '\2')
    Kconv.tosjis(CGI.unescape(trans_text))
  end
end

# puts GoogleTranslate::new("Hello, the world & the space.").translate

class YahooNews

  def initialize uri = nil
    @port = 80
    @proxy = nil
    @proxy_port = nil
    @site = "news.yahoo.com"
    @filtered = Array.new(100)
    if uri
      @news_uri = uri
    else
      @news_uri = "/i/2184/"
    end
  end

  def get
    item_array = Array.new(100)
    res, page = Net::HTTP::new(@site, @port, @proxy, @proxy_port).get(@news_uri)
    div_array = page.split(/<\/p>/m) if page
    div_array.each do |adiv|
      adiv.strip!
      adiv.gsub!(/ +/, ' ')
      if adiv.index("<div class=\"topheadline\">")
        item_array << adiv[adiv.index("<div class=\"topheadline\">") ... adiv.length]
      elsif adiv.index("<li class=\"clearfix\">")
        adiv.split(/<\/div>/).each do |candidate|
          unless candidate.index("<div class=\"ftrimage\">")
            item_array << candidate if candidate.index("<a href=\"/")
          end
        end
      end
    end # div_array.each

    item_array.compact!

    item_array.each do |item|
      uri = String.new
      subtitle = String.new
      description = String.new
      body = String.new
      item.each_line do |line|
        body = body + " " + line.strip
      end
      if body.index("<a href=\"")
        body.sub(/(.*<a href=\")(.+?)(\">)(.+?)(<\/a>)/) {
          uri = $2
          subtitle = $4
        }
        if body.index("<p>")
          description = body.sub(/(.*<p>)(.*)/, '\2')
          @filtered << [uri, subtitle, description]
        end
      end
    end # item_array.each do
    @filtered.compact!
  end # def get
end

class PostSeesaaBlog
  def initialize subject = nil, body = nil, debug_from_file = false
    @post_success = false
    from_address = 'xxxx@yyyyyy.com'
    to_domain = 'hogehoge.foo'
    to_address   = 'hogehoge@hogehoge.foo'
    msg = String.new

    unless debug_from_file
      subject = "Yahoo! News - China (" + Time.now.strftime("%b %d %H:%M") + ")" unless subject
      body = "test test test\nfoo baa foo baa\n" * 5 + "hogehoge hogehoge\n<hr>\n" * 10 unless body
      msg = ["Subject: " + subject, "\n\n", body + "\n" ]
    else
      msg = String.new
      open("msg.txt") { |page|
        msg = page.read
      }
    end

    retry_count = 0
    while retry_count < 5
      retry_count += 1
      begin
        Net::SMTP.start(to_domain, 25) { |smtp|
          smtp.send_mail msg, from_address, to_address
        }
        break
      rescue Timeout::Error
        puts "retry #{retry_count}"
      end
    end
    @post_success = true if retry_count < 5
  end

  def posted?
    @post_success
  end
end

class FilterTest

  def initialize file_name = nil
    @file_name = "./ynsample.html" unless file_name
    @filtered = Array.new(100)
    @body = String.new
  end

  def read
    item_array = Array.new(100)
    f = File::open(@file_name)
    @body = f.read
    f.close
    div_array = @body.split(/<\/p>/m) if @body
    div_array.each do |adiv|
      adiv.strip!
      adiv.gsub!(/ +/, ' ')
      if adiv.index("<div class=\"topheadline\">")
        item_array << adiv[adiv.index("<div class=\"topheadline\">") ... adiv.length]
      elsif adiv.index("<li class=\"clearfix\">")
        adiv.split(/<\/div>/).each do |candidate|
          unless candidate.index("<div class=\"ftrimage\">")
            item_array << candidate if candidate.index("<a href=\"/")
          end
        end
      end
    end # div_array.each

    item_array.compact!

    item_array.each do |item|
      uri = String.new
      subtitle = String.new
      description = String.new
      body = String.new
      item.each_line do |line|
        body = body + " " + line.strip
      end
      if body.index("<a href=\"")
        body.sub(/(.*<a href=\")(.+?)(\">)(.+?)(<\/a>)/) {
          uri = $2
          subtitle = $4
        }
        if body.index("<p>")
          description = body.sub(/(.*<p>)(.*)/, '\2')
          @filtered << [uri, subtitle, description]
        end
      end
    end # item_array.each do
    @filtered.compact!
  end # def read
end

class TestFilterTest
  def initialize
    entry = String.new
    yn = FilterTest.new
    yn.read.each do |uri, sub, desc|
      entry = 
        desc + "<br>" +
        "http://news.yahoo.com" + uri +"<br>" +
        "<blockquote>" + GoogleTranslate::new(sub).translate + "<br>" + 
        GoogleTranslate::new(desc).translate + 
        "</blockquote><hr>"
      PostSeesaaBlog.new sub, entry
    end
  end
end

class TestYahooNewsGoogleTranslate
  def initialize page = nil
    if page
      yn = YahooNews.new("/i/2184/pg:#{page}")
    else
      yn = YahooNews.new
    end
    yn.get.each do |uri, sub, desc|
      entry = "http://news.yahoo.com" + uri +"<br>" +
        desc + "<br>" +
        "<blockquote>" + GoogleTranslate::new(sub).translate + "<br>" + 
        GoogleTranslate::new(desc).translate + 
        "</blockquote><hr>"
#      PostSeesaaBlog.new sub, entry
puts "<h3>" + sub + "</h3>\n"
puts entry
    end
  end
end

# TestYahooNewsGoogleTranslate.new
# TestYahooNewsGoogleTranslate.new(2)
# TestYahooNewsGoogleTranslate.new(3)
# TestYahooNewsGoogleTranslate.new(4)
# TestYahooNewsGoogleTranslate.new(5)
# TestYahooNewsGoogleTranslate.new(6)
# TestFilterTest.new



class YahooNewsRss
  def initialize path_or_url = nil
    if path_or_url
      @url = path_or_url
    else
      @url = "http://rss.news.yahoo.com/rss/oddlyenough"
    end
    @rdf = String.new
    @entries = Array.new(100)
  end

  def get
    open(@url) { |page|
      @rdf = page.read
    }
  end

  def save file_name = nil
    file_name = "index.rdf" unless file_name
    File.open(file_name, "w") {|f|
      f.print(@rdf)
    }
  end

  def parse
    rss = nil
    begin
      rss = RSS::Parser.parse(@rdf, true)
    rescue RSS::InvalidRSSError
      rss = RSS::Parser.parse(@rdf, false)
    end

    rss.items.each do |item|
      if item.description.index("<img src=")
        # ....</a>AFP - Figures due o....</p>
        description = item.description.gsub(/(.*<\/a>)(.+?)(<\/p>.*)/m, '\2')
      else
        description = item.description
      end
      pubdate = ""
      pubdate = item.pubDate if defined? item.pubDate
      @entries << [item.title.tr("\n", ""), description, item.link, pubdate]
    end
    @entries.compact!
  end
end

class FindAlreadyPosted
  def initialize yn, hs
    @yahoo = yn
    @hateshina = hs
  end

  def posted? # if yn.title is in hs.description, true
    bposted = false
    s = @yahoo[0].sub(/(.*)(\(.*(AP|AFP|Reuters).*\))/, '\1')
    s.strip!
#    puts "\ncompare \n" + s + "\n" + @hateshina[1] + "\n--------------\n"
    bposted = true if @hateshina[1].index(s)
    bposted
  end
end

# ynrss = YahooNewsRss.new
# ynrss = YahooNewsRss.new "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/world/asia-pacific/rss.xml"
# ynrss = YahooNewsRss.new "index.rdf"

hsrss = YahooNewsRss.new "http://hateshina.seesaa.net/index.rdf"
hsrss.get
hsentries = hsrss.parse

ynrss = YahooNewsRss.new "http://rss.news.yahoo.com/rss/china"
ynrss.get
ynentries = ynrss.parse

done = false
newentry = Array.new(100)
hsentries.each do |hs|
  newentry.clear
  ynentries.each do |yn|
    unless FindAlreadyPosted.new(yn, hs).posted?
      newentry << yn
    else
      done = true
      break
    end
  end
  break if done
end
newentry.compact!

posting_entry = String.new
unless newentry.empty?
  sub = "Yahoo! News - China (" + Time.now.strftime("%b %d %H:%M") + ")\n"
  newentry.each do |i|
    title_for_translate = i[0].sub(/(.*)(\(.*(AP|AFP|Reuters).*\))/, '\1')
    posting_entry = posting_entry + 
      "<h3>" + i[0] + "</h3>\n" +
      i[1] + "<br>" +
      i[2].sub(/(.*)(http:\/\/news.yahoo.*)/, '\2') + "<br>" +
      "<blockquote>" + GoogleTranslate::new(title_for_translate).translate + "<br>" + 
      GoogleTranslate::new(i[1]).translate + 
      "</blockquote><hr>\n"
  end
  if newentry.size <=2
    ps = PostSeesaaBlog.new nil, posting_entry
    if ps.posted?
      puts "Posted"
    else
      puts sub
      puts posting_entry
    end
  else
    puts sub
    puts posting_entry
  end
end
